package fr.velossity.sample.device.impl;

import java.util.Random;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;

import fr.velossity.sample.device.Device;
import fr.velossity.sample.device.Measurement;
import fr.velossity.sample.device.eventing.MeasurementHandler;

/**
 * 
 * @author C. Saint-Marcel
 * 
 */
public class MockDeviceImpl implements Device, Runnable {

	private String identifier;

	private Thread measureGenerator;

	private BundleContext myContext;

	private Measurement lastMeasurement;

	/*
	 * Default constructor.
	 */
	public MockDeviceImpl(String identifier, BundleContext context) {
		super();
		myContext = context;
		this.identifier = identifier;
	}

	public String getIdentifier() {
		return identifier;
	}

	public String getType() {
		return "Mock";
	}

	/**
	 * Mock device is always functional
	 */
	public String getState() {
		return DEVICE_STATE_FUNCTIONAL;
	}

	public Measurement getLastMeasure() {
		return lastMeasurement;
	}

	public synchronized void activate() {
		if(measureGenerator == null) {
			measureGenerator = new Thread(this);
			measureGenerator.start();
		}
	}

	public synchronized void deactivate() {
		measureGenerator = null;
	}

	public void run() {
		Thread mg = measureGenerator;
		while (measureGenerator == mg) {
			// Regenerate last measurement
			lastMeasurement = new Measurement(MockDeviceImpl.this, new Random().nextFloat()* 1000, "Watt", System.currentTimeMillis());
			
			try {
				// REQUESTS all services implementing MeasurementHandler service
				// Service class name
				String clazz = MeasurementHandler.class.getName();

				ServiceReference sr[] = myContext.getAllServiceReferences(clazz, "(identifier=consumer1*)");
				if (sr != null && sr.length > 0) {
					// BINDS to the consumer
					MeasurementHandler myMeasurementHandler = (MeasurementHandler) myContext
							.getService(sr[0]);
					// NEW DATA notification
					myMeasurementHandler.newMeasure(lastMeasurement);
				}
			} catch (InvalidSyntaxException e) {
				e.printStackTrace();
			}
			try {
				synchronized (this) {
					wait(10000);	
				}				
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

		}

	}

}
